; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
;
;	Project:	2018 512 byte demo entry
;	Region:		PAL
;	Author:		Jeremiah Knol
;	Start Date:	June 17, 2018
;	Finish Date:	July 15, 2018
;
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
;
;	To compile, download DASM and execute the following:
;
;	dasm acidrain.asm -v5 -f3 -oacidrain.bin
;
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
;
;	Enjoy the comments. Hopefully they are more amusing
;	than the ones on YouTube.
;
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

	PROCESSOR 6502

	include "vcs.h"	; v1.05
	
	; download here: https://github.com/munsie/dasm/
	; blob/master/machines/atari2600/vcs.h

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Define RAM Variables - 108 bytes used
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

	SEG.U VARS
	ORG $80

FRAME	ds 1		; the key to this universe
KRNLIN	ds 1		; always keep the Colonel happy
BIT4BF	ds 1		; this one saves a lot of ROM
BUFROW	ds 1		; needed to end everything
TEMP	ds 1		; Fahrenheit, not Celsius
PFBUF0	ds 16		; these are important buffers
PFBUF1	ds 16		; let's hope they don't have bugs
PFBUF2	ds 16		; like the ones that were in Windows
PFBUF3	ds 16		; allowing a simple buffer overflow
COLBUF	ds 16		; to alter sensitive system data
	ORG $100 - 12
TXTFLD	ds 1		; a stadium for text-walking sports
CHRPTR	ds 2		; the only pointer in RAM
RANDOM	ds 1		; where the magic happens
ONEBIT	ds 8		; this one saves a little ROM

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Game Code
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

	SEG CODE	
	ORG $FE00	; 512 byte ROM

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Character Graphics - 89 bytes
;
; Every 4 bytes, a new character starts, but each one uses
; 5 bytes of data, so the last byte of one character is
; shared with the first byte of the next one.
; BasLin uses the low bit of bytes to control the bass.
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

CH____	equ	%00000000
CH___X	equ	%00100010
CH__X_	equ	%01010100
CH__XX	equ	%01110110
CH_X__	equ	%10001000
CH_X_X	equ	%10101010
CH_XX_	equ	%11011100
CH_XXX	equ	%11111110

ChrGfx	.byte CH____	; ' '
	.byte CH____
	.byte CH____
	.byte CH____
BufOfs	.byte CH____	; not a character
	.byte 16
	.byte 32
	.byte 48
BasLin	.byte CH_XX_+1	; '2'
	.byte CH___X+1
	.byte CH_XXX
	.byte CH_X__
	.byte CH__XX	; 'E'
	.byte CH_X__
	.byte CH_XX_+1
	.byte CH_X__+1
	.byte CH__XX	; 'S' / '5'
	.byte CH_X__
	.byte CH_XXX+1
	.byte CH___X
	.byte CH_XX_+1	; 'R'
	.byte CH_X_X+1
	.byte CH_XX_
	.byte CH_X_X
	.byte CH_X_X+1	; 'V'
	.byte CH_X_X+1
	.byte CH_X_X
	.byte CH_XXX
	.byte CH__X_	; 't'
	.byte CH_XXX
	.byte CH__X_
	.byte CH__X_
	.byte CH__X_+1	; 'I' / '1'
	.byte CH__X_+1
	.byte CH__X_
	.byte CH__X_
	.byte CH__X_+1	; 'O'
	.byte CH_X_X
	.byte CH_X_X+1
	.byte CH_X_X
	.byte CH__X_+1	; 'A'
	.byte CH_X_X+1
	.byte CH_X_X
	.byte CH_XXX
	.byte CH_X_X	; 'H'
	.byte CH_X_X
	.byte CH_XXX+1
	.byte CH_X_X+1
	.byte CH_X_X	; 'K'
	.byte CH_X_X
	.byte CH_XX_+1
	.byte CH_X_X
	.byte CH_X_X+1	; 'M'
	.byte CH_XXX+1
	.byte CH_X_X
	.byte CH_X_X
	.byte CH_X_X+1	; 'y'
	.byte CH_X_X+1
	.byte CH_XXX
	.byte CH___X
	.byte CH_XX_	; 'B'
	.byte CH_X_X
	.byte CH_XX_
	.byte CH_X_X
	.byte CH_XX_+1	; 'l'
	.byte CH__X_
	.byte CH__X_+1
	.byte CH__X_
	.byte CH__XX+1	; 'C'
	.byte CH_X__
	.byte CH_X__+1
	.byte CH_X__
	.byte CH__XX	; 'G'
	.byte CH_X__
	.byte CH_X_X
	.byte CH_X_X
	.byte CH__XX	; 'J'
	.byte CH___X
	.byte CH___X
	.byte CH_X_X
	.byte CH_XX_	; 'D'
	.byte CH_X_X
	.byte CH_X_X
	.byte CH_X_X
	.byte CH_XX_	; 'n'
	.byte CH_X_X
	.byte CH_X_X
	.byte CH_X_X
	.byte CH_X_X

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Data Tables - 76 bytes
;
; There are 20 characters, meaning 5 bits are needed to
; represent each one. Here are all the values:
;
;	A=0A	B=0F	C=11	D=14	E=03	G=12	H=0B
;     I/1=08	J=13	K=0C	L=10	M=0D	N=15  O/0=09
;	R=05  S/5=04	T=07	V=06	Y=0E	2=02  SPC=00
;
; Bits D0-D3 are stored in the table Text0F, using 4 bytes
; per line, and bit D4 is stored seperately in the table
; Text10, using 1 byte per line. Both tables are
; upside-down, but read normally, from left to right.
; LedFrq has the audio frequencies used for the lead voice.
;
; When bites are shared between tables, everyone is happy.
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

Text10
	.byte %00000000	; "  SEHR  "
	.byte %01010000	; " DANKE  "
	.byte %00000000	; " BETTER "
	.byte %10010000	; "NTSC IS "
	.byte %00000100	; "AMERICA "
	.byte %00100010	; "MADE IN "
	.byte %10000100	; "GERMANY "
	.byte %00110000	; "HELLO TO"
	.byte %00010100	; "  KNOL  "
	.byte %10000000	; "JEREMIAH"
	.byte %10101000	; "CODED BY"
	.byte %01010001	; "ACIDRAIN"
	.byte %01001000	; "VCS DEMO"
;	.byte %00000000	; "512 BYTE"

Text0F	HEX 0043B500	; "  SEHR  "
	HEX 04A5C300	; " DANKE  "
	HEX 0F377350	; " BETTER "
	HEX 57410840	; "NTSC IS "
	HEX AD3581A0	; "AMERICA "
	HEX DA430850	; "MADE IN "
	HEX 235DA5E0	; "GERMANY "
	HEX B3009079	; "HELLO TO"
	HEX 00C59000	; "  KNOL  "
	HEX 3353D8AB	; "JEREMIAH"
	HEX 194340FE	; "CODED BY"
	HEX A1845A85	; "ACIDRAIN"
	HEX 614043D9	; "VCS DEMO"
	HEX 4820FE;73	; "512 BYTE"

LedFrq	HEX 73 17 1F 17 13 0F 13 17

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; System Clear - 8 bytes
;
; A register = #$00
; X register = #$00
; Addresses $00-$FF (TIA/RAM) = #$00
; SP = #$FF
; Everything else remains in an unknown state
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

SysClr	lda #0		; 2
	tax		; 1
ClrStk	dex		; 1	loop backwards from 255 to 0
	txs		; 1
	pha		; 1	SP will always = X-1
	bne ClrStk	; 2

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Initialization - 15 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

	sec		; 1	generates a table at RAM end
BitTbl	rol		; 1	of all 8 bytes with only one
	pha		; 1	bit set, saving 4 bytes ROM
	bpl BitTbl	; 2	(versus having table in ROM)
	lda #>ChrGfx	; 2
	pha		; 1	set RANDOM's seed
	pha		; 1	set MSB of CHRPTR
	pha		; 1	dec SP past CHRPTR
	lda #29		; 2
	sta AUDC1	; 2
	pha		; 1	set TXTFLD to #29

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Start Frame - 29 bytes
;
; End kernel, do overscan and VSYNC, and start VBLANK timer
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

NxtFrm	ldy #22		; 2
	sty VBLANK	; 2	disable display
	sty AUDC0	; 2	set bass tone
	dey		; 1
	sty CTRLPF	; 2	set PF reflect/priority
	dey		; 1	now Y=#20
	lda #%00001110	; 2	used for VBLANK and VSYNC
OscnLp	sta WSYNC	; 2
	dey		; 1
	bne OscnLp	; 2	output 20 lines of overscan
	ldy #59		; 2	value for TIM64T write
VsncLp	sta WSYNC	; 2
	sty TIM64T	; 3	start VBLANK timer
	sta VSYNC	; 2
	lsr		; 1	shift through VSYNC bits
	bne VsncLp	; 2	output 3 lines of VSYNC

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Logic - 110 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

	lda FRAME	; 2	shift display RAM buffers
	and #%00011111	; 2	only every 32nd frame
	bne Music	; 2	else skip buffer shift

	ldx #78		; 2	do PF and color buffers
	lda PFBUF0+79	; 2	pre-load last value
ShftBf	ldy PFBUF0,x	; 2	get previous value
	sta PFBUF0,x	; 2	overwrite previous value
	tya		; 1	swap to prepare next value
	dex		; 1	step back through buffer
	bpl ShftBf	; 2	keep calm and carry on

	dec BUFROW	; 2	have we finished a section?
	bmi BlnkRo	; 2	if so, add a blank row
	lda TXTFLD	; 2	else get buffer mode
	beq BlnkRo	; 2	handle end of final text
	bmi BlnkRo	; 2	handle after final text
	lsr		; 1	0=random 1=text
	bcc AddRnd	; 2

	tax		; 1
	ldy Text10,x	; 3	get text's D4 data
	sty BIT4BF	; 2	save in RAM
	asl		; 1
	asl		; 1	load X with offset for
	; when you hear this, it's time to turn the page
	tax		; 1	text's 1st character
	ldy BUFROW	; 2
	jsr FwdTxt	; 3	get 2 forward chars
	sta PFBUF0+15	; 2	write to PFBUF0
	jsr RevTxt	; 3	get 2 reversed chars
	sta PFBUF1+15	; 2	write to PFBUF1
	jsr FwdTxt	; 3	etc
	sta PFBUF2+15	; 2	...
	jsr RevTxt	; 3	...
	jmp SetBf3	; 3	no way to not use jmp :/

AddRnd	ldy #3		; 2
RndLp	jsr GetRnd	; 3	get random PF value
	ldx BufOfs,y	; 3	get PF buffer offset
	sta PFBUF0+15,x	; 2	write new PF data
	dey		; 1	next PF buffer
	bpl RndLp	; 2
	bmi Music	; 2

BlnkRo	ldy #5		; 2	BUFROW value for text
	dec TXTFLD	; 2	next text field
	lda TXTFLD	; 2	get text field
	bmi ClrBuf	; 2	clear buffers at end
	asr #%00000011	; 2
	bcs StBfRw	; 2	Y=#5 for text
	tax		; 1
	ldy BufLen,x	; 3	else use BufLen value
StBfRw	sty BUFROW	; 2	set length of next section
ClrBuf	cmp #$F1	; 2
Infnty	beq Infnty	; 2	handle end of program
	lda #0		; 2
	sta PFBUF0+15	; 2	add an empty row
	sta PFBUF1+15	; 2	to the PF buffers
	sta PFBUF2+15	; 2
SetBf3	sta PFBUF3+15	; 2

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Music - 40 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

Music	lax FRAME	; 2	the frame holds the key
	eor #15		; 2	but you must invert it
	asl		; 1	and then divert it
	sta AUDV1	; 2	before you insert it
	txa		; 1
	lsr		; 1
	lsr		; 1
	tax		; 1
	ldy #239	; 2	why are there only 239 beans
	sty KRNLIN	; 2	in an Irish stew, you ask?
	lsr BasLin,x	; 3
	bcs SetVol	; 2	play or mute the bass
	iny		; 1
SetVol	sty AUDV0	; 2	'cuz if there was one more
	asr #14		; 2	it would be too farty!
	tax		; 1
	lda (FRAME-1),y	; 2	yep, bass frequency is
	sta AUDF0	; 2	set to random ROM data
	ldy LedFrq,x	; 3	get lead note
	bit FRAME	; 2
	bmi SetFrq	; 2	offset pitch half the time
	dey		; 1
SetFrq	sty AUDF1	; 2

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Finish Vertical Blank - 5 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

BlnkLp	lda INTIM	; 3	
	bne BlnkLp	; 2	

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Kernel - 55 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

	sta COLUPF	; 2
	sta VBLANK	; 2	enable display
	ldx #15		; 2	draw 16 rows
	lda FRAME	; 2	use frame
	asr #%00011110	; 2	to get offset for
	tay		; 1	height of top row

KRowLp	dey		; 1
	sta WSYNC	; 2 00
	dec KRNLIN	; 2 05	next kernel line
	beq EndKrn	; 2 07	branch out after 240 lines
	tya		; 1 09
	iny		; 1 11
	eor #%00001111	; 2 13
	ora COLBUF,x	; 2 17	get color hue from buffer
	sta COLUPF	; 2 20	set line color
	lda PFBUF0,x	; 2 24
	sta PF1		; 2 27
	lda PFBUF1,x	; 2 31
	sta PF2		; 2 34	assymetric playfield magic
	lda PFBUF3,x	; 2 38
	sta PF1		; 2 41
	lda PFBUF2,x	; 2 45
	sta PF2		; 2 48
	dey		; 1 50	next line in row
	bpl KRowLp	; 2 52
	ldy #15		; 2 54	prepare Y for next row
	dex		; 1 56	next row in kernel
	bpl KRowLp	; 2 58

EndKrn	inc FRAME	; 2
	jmp NxtFrm	; 3	start next frame

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Random Subroutine - 20 bytes
;
; Handle random values
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

GetRnd	lax RANDOM	; 2
	and #$F0	; 2
	sta COLBUF+15	; 2	set row color hue
	txa		; 1
	lsr		; 1	standard Galois LFSR
	bcc NoEor	; 2
	eor #$D4	; 2
NoEor	sta RANDOM	; 2
	asr #%00001110	; 2
	tax		; 1	get value with
	lda ONEBIT,x	; 2	only 1 bit set
	rts		; 1

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Get Forward Text - 14 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

FwdTxt	jsr LftChr	; 3	get left char's graphics
	asr #%11100000	; 2	isolate left char data
	sta TEMP	; 2	save for later

	jsr RgtChr	; 3	get right char's graphics
	asr #%00001110	; 2	isolate right char data
	bcc OrTemp	; 2

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Get Reverse Text - 18 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

RevTxt	jsr LftChr	; 3	get left char's graphics
	asr #%00111000	; 2	isolate mirrored char data
	lsr		; 1	and shift into position
	sta TEMP	; 2	save for later

	jsr RgtChr	; 3	get right char's graphics
	and #%00111000	; 2	isolate mirrored char data
	asl		; 1	and shift
	asl		; 1	into position
OrTemp	ora TEMP	; 2	combine left and right chars
	rts		; 1

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Get Left Character - 10 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

LftChr	lda Text0F,x	; 3	get pair of chars
	asl BIT4BF	; 2	get char's bit 5
	ror		; 1
	asr #%11111000	; 2
	bcc NoOra	; 2	set char pointer

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; Get Right Character - 19 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

RgtChr	lda Text0F,x	; 3
	inx		; 1	prepare X for next call
	and #$0F	; 2	isolate left char
	asl		; 1
	asl		; 1
	asl BIT4BF	; 2	get char's bit 5
	bcc NoOra	; 2
	ora #%01000000	; 2
NoOra	sta CHRPTR	; 2	set char pointer
	lda (CHRPTR),y	; 2	get char graphics
	rts		; 1

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
; End of Cartridge - 4 bytes
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

	ORG $FFFC

	.word SysClr	; reset vector

BufLen	.byte 12	; rows after "even" text phrases
	.byte 5		; rows after "odd" text phrases

; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
;
;	All 512 bytes are accounted for in this demo.
;
;	No NOP's or time-killing absolute addressings of
;	zero-page were used in the making of this demo.
;
;	Plenty of illegal opcodes, though.
;
; <><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
